/**
 * \addtogroup Servo
 * @{
 */

/*****************************************************************************/
/**
 * \file Servo.h
 * Positive Pulse PWM Servo routines for 8051 and Keil C compiler.
 * \author Copyright (c) 2005, Murray R. Van Luyn. <vanluynm@iinet.net.au>
 * \version 0.1
 * \date 28-01-05
 */
/*****************************************************************************/

/*****************************************************************************
 *
 *     THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
 *     OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *     WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *     ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 *     DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *     DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 *     GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 *     INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *     WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *     NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *     SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ******************************************************************************/

#ifndef SERVO_H
#define SERVO_H

#include <REG51.h>

/*****************************************************************************
 *
 *                           Constant Definitions
 *
 *****************************************************************************/

/** 
 * Hardware servo port selection.
 * Adjust \b SERVO_PORT to select servo output port.
 * \hideinitializer
 */
#define SERVO_PORT          P1

/** 
 * Hardware servo port bit enables.
 * Uncomment \b SERVO_x_ENABLE where x corresponds to servo channel to be
 * enabled.
 */
#define SERVO_x_ENABLE    // DO NOT ADJUST - for documentation purposes.

//#define SERVO_0_ENABLE  // These are the servo enable defines to adjust.
//#define SERVO_1_ENABLE
//#define SERVO_2_ENABLE
#define SERVO_3_ENABLE
#define SERVO_4_ENABLE
#define SERVO_5_ENABLE
#define SERVO_6_ENABLE
//#define SERVO_7_ENABLE

/** 
 * Hardware crystal frequency in Hz.
 * Adjust \b XTAL to match hardware.
 * \hideinitializer
 */
#define XTAL                 18432000    

/*
 * Timer 1 mode 1 clock rate.
 */
#define T1_CLOCK             XTAL / 12   

/*
 * Timer 1 reload calculation defines.
 */
#define CYCLES_FOR_OVERFLOW 65535
#define CYCLES_FOR_1MS      0.001  * T1_CLOCK
#define CYCLES_FOR_2MS      0.002  * T1_CLOCK
#define CYCLES_FOR_2MS5     0.0025 * T1_CLOCK

/*
 * Servo position defines.
 */
#define SERVO_POSN_MIN      0
#define SERVO_POSN_MID      127
#define SERVO_POSN_MAX      255

/*
 * Servo port bit defines.
 */
#define SERVO_0             0
#define SERVO_1             1
#define SERVO_2             2
#define SERVO_3             3
#define SERVO_4             4
#define SERVO_5             5
#define SERVO_6             6
#define SERVO_7             7

/*****************************************************************************
 *
 *                           Global Variable Declarations
 *
 *****************************************************************************/

/*
 * Servo port bit declarations.
 */
sbit SERVO_0_OUT = SERVO_PORT^SERVO_0;
sbit SERVO_1_OUT = SERVO_PORT^SERVO_1;
sbit SERVO_2_OUT = SERVO_PORT^SERVO_2;
sbit SERVO_3_OUT = SERVO_PORT^SERVO_3;
sbit SERVO_4_OUT = SERVO_PORT^SERVO_4;
sbit SERVO_5_OUT = SERVO_PORT^SERVO_5;
sbit SERVO_6_OUT = SERVO_PORT^SERVO_6;
sbit SERVO_7_OUT = SERVO_PORT^SERVO_7;

/*
 * Servo position variable declarations.
 */
#ifdef SERVO_0_ENABLE
    extern unsigned char servo_0_posn;
#endif
#ifdef SERVO_1_ENABLE
    extern unsigned char servo_1_posn;
#endif
#ifdef SERVO_2_ENABLE
    extern unsigned char servo_2_posn;
#endif
#ifdef SERVO_3_ENABLE
    extern unsigned char servo_3_posn;
#endif
#ifdef SERVO_4_ENABLE
    extern unsigned char servo_4_posn;
#endif
#ifdef SERVO_5_ENABLE
    extern unsigned char servo_5_posn;
#endif
#ifdef SERVO_6_ENABLE
    extern unsigned char servo_6_posn;
#endif
#ifdef SERVO_7_ENABLE
    extern unsigned char servo_7_posn;
#endif

/*****************************************************************************
 *
 *                           Public Function Prototypes
 *
 *****************************************************************************/
/**
 * PWM Servo routine initialisation function.
 * Initialises the 8051 output port and timer for PWM servo use.
 * Must be called once to set-up hardware during program initialisation.
 * \param -
 * \return -
 */
extern void servo_init(void);
#endif

/** @} */
